@@ -5,7 +5,12 @@ from testing.udir import udir
 
 def create_venv(name):
     tmpdir = udir.join(name)
-    subprocess.check_call(['virtualenv', '-p', sys.executable, str(tmpdir)])
+    try:
+        subprocess.check_call(['virtualenv', '--distribute',
+                               '-p', os.path.abspath(sys.executable),
+                               str(tmpdir)])
+    except OSError as e:
+        py.test.skip("Cannot execute virtualenv: %s" % (e,))
 
     site_packages = None
     for dirpath, dirnames, filenames in os.walk(str(tmpdir)):
@@ -13,19 +18,33 @@ def create_venv(name):
             site_packages = dirpath
             break
     if site_packages:
-        for module in ('pycparser', 'ply'):
-            os.symlink(imp.find_module(module)[1],
-                       os.path.join(site_packages, module))
-
+        try:
+            from cffi import _pycparser
+            modules = ('cffi', '_cffi_backend')
+        except ImportError:
+            modules = ('cffi', '_cffi_backend', 'pycparser')
+            try:
+                import ply
+            except ImportError:
+                pass
+            else:
+                modules += ('ply',)   # needed for older versions of pycparser
+        for module in modules:
+            target = imp.find_module(module)[1]
+            os.symlink(target, os.path.join(site_packages,
+                                            os.path.basename(target)))
     return tmpdir
 
 SNIPPET_DIR = py.path.local(__file__).join('..', 'snippets')
 
 def really_run_setup_and_program(dirname, venv_dir, python_snippet):
-    shutil.rmtree(str(SNIPPET_DIR.join(dirname, 'build')),
-                  ignore_errors=True)
-    shutil.rmtree(str(SNIPPET_DIR.join(dirname, '__pycache__')),
-                  ignore_errors=True)
+    def remove(dir):
+        dir = str(SNIPPET_DIR.join(dirname, dir))
+        shutil.rmtree(dir, ignore_errors=True)
+    remove('build')
+    remove('__pycache__')
+    for basedir in os.listdir(str(SNIPPET_DIR.join(dirname))):
+        remove(os.path.join(basedir, '__pycache__'))
     olddir = os.getcwd()
     python_f = udir.join('x.py')
     python_f.write(py.code.Source(python_snippet))
@@ -48,6 +67,10 @@ def run_setup_and_program(dirname, python_snippet):
         really_run_setup_and_program(dirname, venv_dir, python_snippet)
     finally:
         del sys._force_generic_engine_
+    # the two files lextab.py and yacctab.py are created by not-correctly-
+    # installed versions of pycparser.
+    assert not os.path.exists(str(SNIPPET_DIR.join(dirname, 'lextab.py')))
+    assert not os.path.exists(str(SNIPPET_DIR.join(dirname, 'yacctab.py')))
 
 def test_infrastructure():
     run_setup_and_program('infrastructure', '''
